.include "config.asm"

*=$2801
; ----------------------------------------
; Main menu
; ----------------------------------------
Main
		sei 
		jsr $fd15    ;$fd15 (jmp) - restore default i/o vectors      
		jsr $fda3    ;$fda3 (jmp) - initialize cia & irq             
		jsr $ff5b    ;$ff5b (jmp) - init vic & screen editor         
		cli 
		lda #$00
		sta $d020    ;border color
		sta $d021    ;background color 0
		lda #$0f
		sta $0286	; Current color, cursor color
		jsr $e544	; Clear the screen

; display main menu text
		ldx #$00
-		lda MainMenuText,x
		sta $0400,x
		inx 
		cpx #$78
		bne -	

; keyboard handling
-		lda #$ff
		cmp $d012
		bne -		; wait one frame

		jsr $ffe4    ;$ffe4 - get a byte from channel          
		cmp #$20     ;Space pressed?
		beq JPlayNote
		cmp #$85     ;F1 pressed?
		beq JEditNote

; Timer, wait a little then going directly to the PlayNote
		inc	MT+1
MT		lda #$00
		bne -
		jmp JPlayNote

JPlayNote	jmp PlayNote
JEditNote	jmp EditNote

MainMenuText
.text "          lethargy noter v1.0b          "
.text "  press <f1> to write another note...   "
.text "      press space to read note...       "


;---------------------------------------
;- Note editor
;---------------------------------------
EditNote	
; Editor init	
		sei 
		jsr $fd15    ;$fd15 (jmp) - restore default i/o vectors      
		jsr $fda3    ;$fda3 (jmp) - initialize cia & irq             
		jsr $ff5b    ;$ff5b (jmp) - init vic & screen editor         
		lda #$00
		sta $d020    ;border color
		sta $d021    ;background color 0
		lda #$0f
		sta $0286	; Current color, cursor color
		jsr $e544	; clear the screen
		lda #$00
		sta $d418    ;select filter mode and volume
		cli 

; Display editor text
		ldx #$00
-		lda EditMenuText,x
		sta ScreenRAM,x
		inx 
		bne -

; Keyboard handling
-		jsr $ffe4    ;$ffe4 - get a byte from channel
		cmp #$4c     	; L - Load Music
		beq ELoadMusicJ
		cmp #$54     	; T - Test Note
		beq ETestNoteJ
		cmp #$53     	; S - Save Note
		beq ESaveNoteJ
		cmp #$45     	; E - Edit Note ;#%01000101
		beq EEditNoteJ
		cmp #$43     	; C - Load Chharset
		beq ELoadCharsetJ
		cmp #$44     	; D - Directory ;#%01000100
		beq EDirectoryJ
		jmp -

ESaveNoteJ		jmp ESaveNoteJJ
ELoadCharsetJ	jmp ELoadCharset
ELoadMusicJ		jmp ELoadMusic
ETestNoteJ		jmp ETestNote
EDirectoryJ		jmp EDirectory
EEditNoteJ		jmp EEditNoteJJ


; ----------------------------------------
; reading number (byte or word) from screen (on the address $0405)
; and converting from on-screen address value to byte or word (two modes can be set) 
; ----------------------------------------
InputNumberfromScreen
		lda #<ScreenRAM+5
		sta RHStore+1
		lda #>ScreenRAM+5
		sta RHStore+2
InputNumberfromScreen2
; input, based on the "InputMode"
		lda InputMode
		cmp #$01
		beq +
		jsr ReadHexaCharsWord
+		lda InputMode
		cmp #$00
		beq +
		jsr ReadHexaCharsByte
+		
; convert onscreen char #1
		lda ScreenIn1
		jsr INConv
		asl
		asl
		asl
		asl
		sta InputNumberResultHI
; convert onscreen char #2
		lda ScreenIn2
		jsr	INConv
		clc 
		adc InputNumberResultHI
		sta InputNumberResultHI
; convert onscreen char #3
		lda ScreenIn3
		jsr INConv
		asl
		asl
		asl
		asl
		sta InputNumberResultLO
; convert onscreen char #4
		lda ScreenIn4
		jsr INConv
		clc 
		adc InputNumberResultLO
		sta InputNumberResultLO
		rts 

INConv	and #%00111111
		cmp #$30
		bcc +
		sec 
		sbc #$30
		rts
+		clc 
		adc #$09
		rts 

ScreenIn1	.byte $00
ScreenIn2	.byte $00
ScreenIn3	.byte $00
ScreenIn4	.byte $00
InputNumberResultLO		.byte $00	; conversion address result: LO
InputNumberResultHI		.byte $00	; conversion address result: HI ?
InputMode	.byte $00	; input/conversion mode: $00 (WORD) / $01 (BYTE)

;---------------------------------------
; Load Charset menu item
;---------------------------------------
ELoadCharset	
		jsr $e544	; clear the screen

; Get the charset name
		ldx #$00
-		jsr $ffcf	;$ffcf - input character                  
		sta LoadCharsetFilename,x
		inx 
		cmp #$0d	; Charset filename input
		bne -

; Clear the screen with black characters
		lda #$00
		sta $0286	; Current color, cursor color
		jsr $e544	; clear the screen

; Load the file to the address of $0800
		lda #$01
		ldx #$08
		ldy #$00
		jsr $ffba    ;$ffba - set file parameters              
		lda #$0f
		ldx #>LoadCharsetFilename
		ldy #<LoadCharsetFilename
		jsr $ffbd    ;$ffbd - set file name                    
		lda #$00
		ldx #$00
		ldy #$08	; loading to address $0800
		jsr $ffd5    ;$ffd5 - load after call setlfs,setnam    

; Clear the screen with grey characters and return to the Edit Menu
		lda #$0f
		sta $0286	; Current color, cursor color
		jsr $e544	; clear the screen
		jmp EditNote 	; return back to the main NoteEditor screen

LoadCharsetFilename
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00

;---------------------------------------
; Load Music menu item
;---------------------------------------
ELoadMusic	
		jsr $e544	; clear the screen

; filename input
		ldx #$00
-		jsr $ffcf    ;$ffcf - input character                  
		sta LoadMusicFilename,x
		inx 
		cmp #$0d	; Music filename input
		bne -

; Clear the screen with black characters
		lda #$00
		sta $0286	; Current color, cursor color
		jsr $e544	; clear the screen

; loading the music file to the address of $1000
		lda #$01
		ldx #$08	; ",8,1"
		ldy #$00
		jsr $ffba    ;$ffba - set file parameters
		lda #$0f     ;#%00001111
		ldx #>LoadMusicFilename
		ldy #<LoadMusicFilename
		jsr $ffbd    ;$ffbd - set file name                    
		lda #$00
		ldx #$00
		ldy #$10	; loading to address $1000
		jsr $ffd5    ;$ffd5 - load after call setlfs,setnam    

; Clear the screen with gray characters
		lda #$0f
		sta $0286	; Current color, cursor color
		jsr $e544	; clear the screen

; write on screen: "init?"
		ldx #$00
-		lda TextINIT,x
		sta $0400,x
		inx 
		cpx #$05
		bne -

; read, convert and store the two byte result (e.g. $1000)
		lda #$00
		sta InputMode
		jsr InputNumberfromScreen	; from screen converter
		lda InputNumberResultLO
		sta MusicInit+1
		lda InputNumberResultHI
		sta MusicInit+2

; write on screen: "play?"
		jsr $e544	; clear the screen
		ldx #$00
-		lda TextPLAY,x
		sta $0400,x
		inx
		cpx #$05
		bne -

		lda #$00
		sta InputMode
		jsr InputNumberfromScreen
		lda InputNumberResultLO
		sta MusicPlay+1
		lda InputNumberResultHI
		sta MusicPlay+2

		lda #<ScreenRAM+5
		sta RBStore+1
		lda #>ScreenRAM+5
		sta RBStore+2

; write on screen: "lda#$"
		jsr $e544	; clear the screen
		ldx #$00
-		lda TextLDA,x
		sta $0400,x
		inx 
		cpx #$05
		bne -

		lda #$01
		sta InputMode
		jsr InputNumberfromScreen2
		lda InputNumberResultHI
		sta MusicInit0+1
		jmp EditNote	; return back to the main NoteEditor screen

LoadMusicFilename	
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00

TextINIT	.text "init?"
TextPLAY	.text "play?",$20,$20
TextLDA		.text "lda#$",$20

;---------------------------------------
; Read hexa WORD (#0-#9 + a-f)
;---------------------------------------
ReadHexaCharsWord
		jsr $ffe4    ;$ffe4 - get a byte from channel  
		cmp #$30		; "0"
		beq RHCharProcess
		cmp #$31		; "1"
		beq RHCharProcess
		cmp #$32		; "2"
		beq RHCharProcess
		cmp #$33		; "3"
		beq RHCharProcess
		cmp #$34		; "4"
		beq RHCharProcess
		cmp #$35		; "5"
		beq RHCharProcess
		cmp #$36		; "6"
		beq RHCharProcess
		cmp #$37		; "7"
		beq RHCharProcess
		cmp #$38		; "8"
		beq RHCharProcess
		cmp #$39		; "9"
		beq RHCharProcess
		cmp #$41		; "a"
		beq RHCharProcess
		cmp #$42		; "b"
		beq RHCharProcess
		cmp #$43		; "c"
		beq RHCharProcess
		cmp #$44		; "d"
		beq RHCharProcess
		cmp #$45		; "e"
		beq RHCharProcess
		cmp #$46		; "f"
		beq RHCharProcess
		cmp #$0d		; "Return"
		beq RHCharReturn
		cmp #$14		; "Del"
		beq RHCharDel
		jmp ReadHexaCharsWord

RHCharProcess
		ldx RHStore+1
		cpx #$09
		beq RHReachedMax	; if reached the $0409 then move back
		cmp #$41
		bpl RHProcessAlfa	; if <="A" then process A-F characters
RHStore	sta ScreenRAM+5
		inc RHStore+1
		jmp ReadHexaCharsWord

RHProcessAlfa	; processing characters A-F
		clc
		adc #$c0     ; convert to screencode and store on screen
		jmp RHStore

RHReachedMax	; reached the maximum position? then lets go back one char
		dec RHStore+1
		jmp ReadHexaCharsWord

RHCharReturn	
		lda RHStore+1
		cmp #$08		; if not yet all 4 chars are filled out then read another one
		bpl RHReturnData
		jmp ReadHexaCharsWord

RHCharDel	
		ldx RHStore+1
		cpx #$07
		beq +				; if at $0407 then...
		lda #$20
		sta ScreenRAM+$28
+		dex
		stx RHD+1
		lda #$20
RHD		sta ScreenRAM		; set the previous screen character and place a SPACE there
		dec RHStore+1		; moving back the cursor
		lda RHStore+1
		cmp #$04
		beq +				; if cursor is at $0404 than don't move back :)
		jmp ReadHexaCharsWord
+		inc RHStore+1
		jmp ReadHexaCharsWord

RHReturnData	
		lda ScreenRAM+5
		sta ScreenIn1
		lda ScreenRAM+6
		sta ScreenIn2
		lda ScreenRAM+7
		sta ScreenIn3
		lda ScreenRAM+8
		sta ScreenIn4
		rts					; return the 4 on screen characters when pressing RETURN :) 

;---------------------------------------
; Read hexa BYTE (#0-#9 + a-f)
;---------------------------------------
ReadHexaCharsByte
		jsr $ffe4    ;$ffe4 - get a byte from channel          
		cmp #$30     ; "0"
		beq RBCharProcess
		cmp #$31     ; "1"
		beq RBCharProcess
		cmp #$32     ; "2"
		beq RBCharProcess
		cmp #$33     ; "3"
		beq RBCharProcess
		cmp #$34     ; "4"
		beq RBCharProcess
		cmp #$35     ; "5"
		beq RBCharProcess
		cmp #$36     ; "6"
		beq RBCharProcess
		cmp #$37     ; "7"
		beq RBCharProcess
		cmp #$38     ; "8"
		beq RBCharProcess
		cmp #$39     ; "9"
		beq RBCharProcess
		cmp #$41     ; "a"
		beq RBCharProcess
		cmp #$42     ; "b"
		beq RBCharProcess
		cmp #$43     ; "c"
		beq RBCharProcess
		cmp #$44     ; "d"
		beq RBCharProcess
		cmp #$45     ; "e"
		beq RBCharProcess
		cmp #$46     ; "f"
		beq RBCharProcess
		cmp #$0d     ; "RETURN"
		beq RBCharReturn
		cmp #$14     ; "DEL"
		beq RBCharDel
		jmp ReadHexaCharsByte

RBCharProcess	
		ldx RBStore+1
		cpx #$07
		beq RBReachedMax	; if reached the $0407 then move back
		cmp #$41
		bpl RBProcessAlfa	; if <="A" then process A-F characters
RBStore	sta ScreenRAM+5
		inc RBStore+1
		jmp ReadHexaCharsByte

RBProcessAlfa	; processing characters A-F
		clc
		adc #$c0     ; convert to screencode and store on screen
		jmp RBStore

RBReachedMax		; reached the maximum position? then lets go back one char
		dec RBStore+1
		jmp ReadHexaCharsByte

RBCharReturn
		lda RBStore+1
		cmp #$06	; if not yet all 2 chars are filled out then read another one
		bpl RBReturnData
		jmp ReadHexaCharsByte

RBCharDel
		ldx RBStore+1
		cpx #$07		; if at $0407 then...
		beq +
		lda #$20
		sta ScreenRAM+$28
+		dex 
		stx RBD+1
		lda #$20
RBD		sta ScreenRAM	; set the previous screen character and place a SPACE there
		dec RBStore+1	; moving back the cursor
		lda RBStore+1
		cmp #$04		; if cursor is at $0404 than don't move back :)
		beq +
		jmp ReadHexaCharsByte
+		inc RBStore+1
		jmp ReadHexaCharsByte

RBReturnData	
		lda ScreenRAM+5
		sta ScreenIn1
		lda ScreenRAM+6
		sta ScreenIn2
		lda ScreenRAM+7
		sta ScreenIn3
		lda ScreenRAM+8
		sta ScreenIn4
		rts 				; return the 4 on screen characters when pressing RETURN :)


;---------------------------------------
; Directory menu item - pull down a directory list and display on an empty screen
;---------------------------------------
EDirectory
		jsr $e544	; clear the screen
		lda $ba
		jsr $ffb1    ;$ffb1 - make serial device listen        
		lda #$f0
		jsr $ff93    ;$ff93 - send secondary addr after listen 
		lda #$24
		jsr $ffa8    ;$ffa8 - output byte to serial            
		jsr $ffae    ;$ffae - unlisten all serial devices      
		lda $ba
		jsr $ffb4    ;$ffb4 - make serial device talk          
		lda #$60
		jsr $ff96    ;$ff96 - send secondary addr after talk   
		lda #$00
		sta $90
		ldy #$03
ED1		sty $b7
		jsr $ffa5    ;$ffa5 - input byte from serial           
		sta $c3
		jsr $ffa5    ;$ffa5 - input byte from serial           
		sta $c4
		ldy $90
		bne ED2
		ldy $b7
		dey 
		bne ED1
		lda $91
		cmp #$7f
		beq ED3
		ldx $c3
		lda $c4
		jsr $bdcd		; basic call
		lda #$20
		jsr $ffd2		;$ffd2 - output character                 
		lda #$00
ED2		bne ED3
-		jsr $ffa5    ;$ffa5 - input byte from serial           
		beq +
		jsr $ffd2    ;$ffd2 - output character                 
		bne -
+		lda #$0d
		jsr $ffd2    ;$ffd2 - output character                 
		ldy #$02
		bne ED1
ED3		jsr $ffab    ;$ffab - untalk all serial devices        
		lda $ba
		jsr $ffb1    ;$ffb1 - make serial device listen        
		lda #$e0
		jsr $ff93    ;$ff93 - send secondary addr after listen 
		jsr $ffae    ;$ffae - unlisten all serial devices      
-		lda $dc01    ;cia1: data port register b
		cmp #$ef     ;#%11101111
		bne -
		jsr $e544	; clear the screen
		jmp EditNote	; return back to the main NoteEditor screen


;---------------------------------------
;- Test Note menu item
;---------------------------------------
ETestNote	
		lda #$01
		sta NoteTestFlag
		jmp PlayNote


;---------------------------------------
;- Edit Note menu item
;---------------------------------------
EEditNoteJJ
		jmp EEditNote

; Note Editor
EEditNote
		sei
		jsr $fd15    ;$fd15 (jmp) - restore default i/o vectors      
		jsr $fda3    ;$fda3 (jmp) - initialize cia & irq             
		jsr $ff5b    ;$ff5b (jmp) - init vic & screen editor         
		cli 
		lda #$40     ;Set default screen pointers to $4000
		sta SourceScreenStartHI
		sta ScreenBackcopyHI
		lda #$44     ;...and to $4400
		sta SourceScreenEndHI
ReturnEditPage	
		jsr EditPage
		lda ScreenpointerSelector
		cmp #$00		;#$00 - End editing
		beq EndNoteEditing
		cmp #$01		;#$01 - Next Page
		beq NextPage
		cmp #$02		;#$02 - Previous Page
		beq PreviousPage
		jmp EditNote	; return back to the main NoteEditor screen

NextPage	; updating the screen pointers with +$0400
		lda SourceScreenStartHI
		clc
		adc #$04
		sta SourceScreenStartHI
		sta ScreenBackcopyHI
		clc
		adc #$04
		sta SourceScreenEndHI
		jmp ReturnEditPage

PreviousPage	; updating the screen pointers with +$0400
		lda SourceScreenStartHI	
		cmp #$40
		beq +				; do nothing if already at $4000
		clc 
		adc #$fc			; practically -#$04 :)
		sta SourceScreenStartHI
		sta ScreenBackcopyHI
		clc 
		adc #$04
		sta SourceScreenEndHI
		jmp ReturnEditPage
+		jmp ReturnEditPage

EndNoteEditing	
		lda SourceScreenStartHI
		clc 
		adc #$04     ;#%00000100
		sta ENE+2
		lda #$7a		; $7a on the next screen in RAM means: end of note
ENE		sta $4400
		jmp EditNote	; return back to the main NoteEditor screen

EditPage
		lda #$0b
		sta $d020    ;border color
		lda #$0c
		sta $d021    ;background color 0
		lda #$00
		sta $0286	; Current color, cursor color
		jsr $e544	; clear the screen
		lda #%00010010
		sta $d018    ;vic memory control register
		lda #$93
		jsr $ffd2    ;$ffd2 - output character                 
		ldx #$00	; copy current pointed (default:$4000) screen to $0400
		jsr CopyOneScreen
		ldx #$06	; do another copy from $C300 to $C300 -> ? this seems to be unnecessary ?
		jsr CopyOneScreen
		jsr EditPageKeys ; actual page editor routine
		ldx #$0c	; copy the modified page back from $0400 to memory (default:$4000)
		jsr CopyOneScreen
		lda #$92	; sending $92	reverse off command to the screen (to turn off cursor ;)
		jsr $ffd2    ;$ffd2 - output character             
		rts			; page edit done


; this routine copies one screen content within the RAM, input: X -> ScreenRAMPointers
CopyOneScreen
; copy 6 bytes to $f7 zeropage location (X is an input!)
		ldy #$00
-		lda ScreenPointers,x
		sta $00f7,y
		inx 
		iny 
		cpy #$06
		bne -
; Display/Copy back the current page (e.g. copy RAM content from $4000 to $0400)
		ldy #$00
-		lda ($f7),y
		sta ($f9),y
		inc $f7
		bne +
		inc $f8
+		inc $f9
		bne +
		inc $fa
+		lda $f8
		cmp $fc
		bne -
		lda $f7
		cmp $fb
		bne -
; Clean out the ZP Screen RAM pointers table
		ldx #$05
		lda #$00
-		sta $f7,x
		dex 
		bpl -
		rts 

; teh actual page editor with reading characters from IO through KERNAL routines
EditPageKeys	
		lda #$13	; sending a $13	Home to the screen (home the cursor ;)
		jsr $ffd2    ;$ffd2 - output character
-		jsr InvertCurrentChar

; Read one character
-		jsr $ffe4    ;$ffe4 - get a byte from channel          
		cmp #$00     ;#%00000000
		beq -

		cmp #$87		; "F5" - End NoteEdit
		beq EPKEndEdit
		cmp #$85		; "F1" - Next Page
		beq EPKNextPage
		cmp #$86		; "F3" - Previous Page
		beq EPKPreviousPage
		tax
		jsr InvertCurrentChar
		txa
		jsr $ffd2    ;$ffd2 - output character / actually write the inverted char to screen!
		clc 
		bcc --			; CLC/BCC combination means always jump ;) 

ReturnFromEdit	
		jsr InvertCurrentChar
		rts

; Inverting the character under the current column ($d1) and line ($d1-$d2)
InvertCurrentChar	
		ldy $d3
		lda ($d1),y
		eor #$80
		sta ($d1),y
		rts

EPKEndEdit	
		lda #$00
		sta ScreenpointerSelector
		jmp ReturnFromEdit
EPKNextPage
		lda #$01
		sta ScreenpointerSelector
		jmp ReturnFromEdit
EPKPreviousPage	
		lda #$02
		sta ScreenpointerSelector
		jmp ReturnFromEdit

; 3 set of pointers (6 bytes each)
; Source Start Address/ Target Address / Source End Address
ScreenPointers
; Set #1 -> $4000 / $0400 / $4400
					.byte $00
SourceScreenStartHI	.byte $40		; SourceStartAddressHI
					.byte $00
					.byte $04
					.byte $00
SourceScreenEndHI	.byte $44
; Set #2 -> $c300 / $c300 / $c400
					.byte $00
					.byte $c3
					.byte $00
					.byte $c3
					.byte $00
					.byte $c4
; Set #3 -> $0400 / $4000 / $07ff
					.byte $00
					.byte $04
					.byte $00
ScreenBackcopyHI	.byte $40
					.byte $ff
					.byte $07

ScreenpointerSelector	.byte $00

ESaveNoteJJ	jmp ESaveNote

EditMenuText
.text "          lethargy noter v1.0b          "
.text "                                        "
.text "    l - load music    t - test note     "
.text "    s - save note     e - edit note     "
.text "    c - load charset  d - directory     "
.text "                                        "
.text "(c)lethargy 1992",$00



;---------------------------------------
;- Play the note
;---------------------------------------
PlayNote	
; IRQ setup + preparations
		sei
		lda #<IRQ1
		sta $0314    ;irq
		lda #>IRQ1
		sta $0315    ;irq
		lda #$32
		sta $d012    ;raster position
		lda #$81
		sta $d01a    ;vic interrupt mask register (imr)
		lda #$7f
		sta $dc0d    ;cia1: cia interrupt control register
		jsr MusicInit0

		ldx #$00
		lda #$00
-		sta TempRasterTable,x
a2f2c	inx 
		bne -		; clean out the temporary RasterTable RAM area

		lda #$00
		sta $0286	; Current color, cursor color
		jsr $e544	; clear the screen

		lda #$12
		sta $d018    ;vic memory control register
		cli 

DisplayPage	
		lda #$0f
		sta $d021    ;background color 0

; set page pointers
		lda PagePoint1+2
		clc
		adc #$04
		sta PagePoint1+2
		lda PagePoint2+2
		clc 
		adc #$04
		sta PagePoint2+2
		lda PagePoint3+2
		clc 
		adc #$04
		sta PagePoint3+2
		lda PagePoint4+2
		clc 
		adc #$04
		sta PagePoint4+2
		lda PagePoint5+2
		clc 
		adc #$04
		sta PagePoint5+2

; Display current noter page
		ldx #$00
PagePoint1		
-		lda NoterPage1-$0400,x
		sta ScreenRAM,x
PagePoint2
		lda NoterPage1-$0300,x
		sta ScreenRAM+$100,x
PagePoint3
		lda NoterPage1-$0200,x
		sta ScreenRAM+$200,x
PagePoint4
		lda NoterPage1-$0100,x
		sta ScreenRAM+$300,x
		inx 
		bne -

		jsr SetRasterMoveDown

-		lda RasterTrigger
		cmp #$00
		bne -				; wait while the raster moving is finished

KeyboardCheck
		lda #$00
		sta	$dc00
		lda $dc01    ;cia1: data port register b
		cmp #%01111111
		beq RunStopPressed
		cmp #%11101111
		bne KeyboardCheck

		jsr SetRasterMoveUp

-		lda RasterTrigger
		cmp #$00
		bne -				; wait while the raster moving is finished

PagePoint5
		lda NoterPage1
		cmp #$7a				; "end of note" sign -> next page, first char is $7a (#122)
		beq ResetToFirstPage
		jmp DisplayPage

ResetToFirstPage
		lda #$3c
		sta PagePoint1+2
		lda #$3d
		sta PagePoint2+2
		lda #$3e
		sta PagePoint3+2
		lda #$3f
		sta PagePoint4+2
		lda #$40
		sta PagePoint5+2
		jmp DisplayPage

RunStopPressed	
		lda NoteTestFlag
		cmp #$01
		bne KeyboardCheck
		jmp ReturnToEditor

; ----------------------------------------
; IRQ1 - background raster
; ----------------------------------------

IRQ1	nop
		nop
		ldx #$00
-		lda TempRasterTable,x
		sta $d021    ;background color 0
		sta $d021    ;background color 0
		ldy RasterDelayTable,x
-		dey 
		bne -		; raster delay, based on the table at $3f00
		inx
		cpx #$c8
		bne --

		lda #$32
		sta $d012    ;raster position
		lda #$1b
		sta $d011    ;vic control register 1
		inc $d019    ;vic interrupt request register (irr)
		jsr RasterMove
		jsr MusicPlay
		jmp $ea7a

SetRasterMoveDown	
		lda #$01
		sta RasterTrigger
		rts

SetRasterMoveUp	
		lda #$02
		sta RasterTrigger
		rts

RasterMove	
		lda RasterTrigger
		cmp #$01
		beq RasterMoveDown
		cmp #$02
		beq RasterMoveUp
		rts 

RasterMoveDown
		jsr RasterStateCopy
		lda RSCC+1
		cmp #$60
		beq RasterMoveCompleted
		rts

RasterMoveCompleted
		lda #$00
		sta RasterTrigger
		rts 

RasterMoveUp	
		jsr RasterStateCopy
		lda RSCC+1
		cmp #$20
		beq RasterMoveCompleted
		rts 

RasterStateCopy	
		ldx #$00
RSCC	lda RasterSinus
		clc
		adc #$c8
		tay
-		lda RasterColors,x
		sta TempRasterTable,y
		inx
		iny
		cpx #$38
		bne -
		inc RSCC+1
		lda RSCC+1
		cmp #$80
		beq +
		rts
+		lda #$00
		sta RSCC+1
		rts 

RasterTrigger	.byte $01		; $00=no raster move  $01=raster moves down  $02=raster moves up

NoteTestFlag	.byte $00

MusicInit0	lda #$00
MusicInit	jmp $18ea
MusicPlay	jmp $14ea

ReturnToEditor	
		lda #$01
		sta RasterTrigger		; reset to default
		lda #<RasterSinus
		sta RSCC+1
		lda #>RasterSinus
		sta RSCC+2				; reset the RasterStateCopy pointers
		lda #$00
		sta NoteTestFlag
		lda #$3c
		sta PagePoint1+2
		lda #$3d
		sta PagePoint2+2
		lda #$3e
		sta PagePoint3+2
		lda #$3f
		sta PagePoint4+2
		lda #$40
		sta PagePoint5+2
		jmp EditNote	; return back to the main NoteEditor screen


;---------------------------------------
; Save Note menu item
;---------------------------------------
ESaveNote

; Initialize system and set background/border to black
		sei
		jsr $fd15    ;$fd15 (jmp) - restore default i/o vectors
		jsr $fda3    ;$fda3 (jmp) - initialize cia & irq
		jsr $ff5b    ;$ff5b (jmp) - init vic & screen editor
		cli
		lda #$00
		sta $d020    ;border color
		sta $d021    ;background color 0
		lda #$0b
		sta $0286	; Current color, cursor color
		jsr $e544	; clear the screen

; Copy the compressor and save routines to the area of $03b5-$07ff
		ldx #$00
-		lda CompressSave,x
		sta $03b5,x
		lda CSPoint,x
		sta ScreenRAM,x
		lda CSPoint+$100,x
		sta ScreenRAM+$100,x
		lda CSPoint+$200,x
		sta ScreenRAM+$200,x
		lda CSPoint+$300,x
		sta ScreenRAM+$300,x
		inx
		bne -
		jmp $03b5	 ; launch the Compress and Save routine


; ----------------------------------------
; Compressor and save routine (originally at $31b5, copied to $03b5)
; Note: this part is untouched, and left as it was originally
; ----------------------------------------
CompressSave
		ldx #$18			; $31b5 -> $03b5
		ldy #$0f
		lda #$10
		sta $0682
		lda #$3c
		sta $06d1
		jsr $051f

; Open file for writing...
		lda $0683		; length of the filename
		ldx #$d2
		ldy #$06		; filename is located at $06d2
		jsr $ffbd    ;$ffbd - set file name                    
		lda #$01
		tay 
		ldx #$08
		jsr $ffba    ;$ffba - set file parameters              
		jsr $ffc0    ;$ffc0 - open log.file after setlfs,setnam
		ldx #$01
		jsr $ffc9    ;$ffc9 - open channel for output          

; save the load address to the file ($0801)
		lda #$01
		jsr $ffd2    ;$ffd2 - output character                 
		lda #$08
		jsr $ffd2    ;$ffd2 - output character                 

		sei
		lda #$34
		sta $01
		lda #$00
		tay 
-		sta CompressTempArea,y		; $31f2 -> $03f2
		dey
		bne -

		ldx #$08
		ldy #$00
		stx $af
		stx $fc
CSPoint	stx $0636			; CSPoint: $3200 -> $0400 
		sty $ae
		sty $0632
		sty $fb
		ldx #>$d000
		ldy #<$d000
		stx $2e
		sty $2d			; pointers to the maximum RAM location until the compressor works?

-		ldy #$00
		lda ($ae),y
		tax 
		clc
		inc CompressTempArea,x
		bne +
		lda #$ff
		sta CompressTempArea,x
+		clc 
		inc $ae
		bne +
		inc $af
+		lda $ae
		cmp $2d
		bne -
		lda $af
		cmp $2e
		bne -

		jsr $04fa
		sty $03
		sty $063c
		inx
		jsr $04fc
		sty $04
		sty $0652

		lda #$37
		sta $01

		jsr $05c5		; call writing the decomp header to disk (?)

		sei 
		lda #$34
		sta $01
		lda $ae
		bne +
		dec $af
+		dec $ae					; $3258 -> $0458
		ldy #$00
		lda ($ae),y
		sta $06e3
		tax
		inx
		txa
		iny
		sta ($ae),y
		inc $ae
		bne b326d
		inc $af
b326d	lda $fb					; $326d -> $046d
		cmp $ae
		bne +
		lda $fc
		cmp $af
		bne +
		jmp $04e0

+		ldy #$00				; $327c -> $047c
		lda ($fb),y
		iny 
		cmp ($fb),y
		bne b32c4
		iny 
		cmp ($fb),y
		bne b32c4
		iny 
		cmp ($fb),y
		bne b32c4
-		iny 					; $328f -> $048f
		beq +
		cmp ($fb),y
		beq -
+		cmp #$00				; $3296 -> $0496
		beq b32b8
		pha 
		lda $04
		jsr $050c
		tya 
		jsr $050c
		pla 
		jsr $050c
		clc 
		tya 
		beq +
		adc $fb
		sta $fb
		bcc b326d
+		inc $fc		; $32b2 -> $04b2
		clc
		jmp $046d

b32b8	lda $03		; $32b8 -> $04b8
		jsr $050c
		tya
		jsr $050c
		jmp $04a8

b32c4	cmp $04		; $32c4 -> $04c4
		beq +
		cmp $03
		beq +
		jsr $050c
		jmp $04d7
+		ldy #$00	; $32d2 -> $04d2
		jmp $048f

		inc $fb
		bne +
		inc $fc
+		jmp $046d		; $32dd -> $04dd

		lda #$37		; $04e0 (?)
		sta $01
		cli
		jsr $f333    ;$f333 (jmp) - restore default devices          
		lda #$01
		jsr $ffc3    ;$ffc3 - close a logical file             
		ldy #$00
		lda $06e3
		sta ($ae),y
		jmp $fce2		; end of compress and save, now time to call hardware reset

		jmp $fce3		; ??? seems to be a bug. but should be left here ;) 

		ldx #$00
		txa
		ldy #$00
-		cmp CompressTempArea,y	; $32ff -> $04ff
		beq +
		iny 
		bne -
		inx 
		jmp $04fc
+		rts 

		sty $02
		ldy #$37
		sty $01
		jsr $ffd2    ;$ffd2 - output character                 
		sei
		ldy #$34
		sty $01
		ldy $02
		inc $34
		rts

		jsr $e50c
		lda #$00
		sta $0683
		jsr $05bc
b332a	jsr $ffe4    ;$ffe4 - get a byte from channel          
		beq b332a
		ldx #$00
-		cmp $0684,x
		beq +
		inx 
		cpx $06d1
		bne -
		jmp $052a

+		cmp #$14		; $333f -> $053f
		bne b336d
		lda $0683
		beq b332a
		dec $0683
		jsr $05bc
		lda #$9d
		jsr $ffd2    ;$ffd2 - output character           
		ldy #$30
		lda $06d1
		cmp #$12
		beq +
		ldy #$20
+		tya 
		jsr $ffd2    ;$ffd2 - output character                 
		lda #$9d     ;#%10011101
		jsr $ffd2    ;$ffd2 - output character                 
		jsr $05bc
		jmp $052a

b336d	cmp #$0d		; $336d -> $056d
		bne +
		jsr $05bc
		rts 

+		ldy $0683		; $3375 -> $0575
		cpy $0682
		beq b332a
		tax 
		jsr $05bc
		txa 
		ldy $0683
		sta $06d2,y
		jsr $ffd2    ;$ffd2 - output character                 
		inc $0683
		jsr $05bc
		jmp $052a

		lda $0770,y
		cmp #$30
		bcs +
		and #$0f
		clc 
		adc #$09
+		and #$0f			; $33a0 -> $05a0
		asl
		asl
		asl
		asl			;.byte $0a,$0a,$0a,$0a
		sta $06c0
		iny
		lda $0770,y
		cmp #$30
		bcs +
		and #%00001111
		clc 
		adc #$09
+		and #%00001111		; $33b6 -> $05b6
		ora $06c0
		rts

		ldy $d3				; $33bc -> $05bc
		lda ($d1),y
		eor #$80
		sta ($d1),y
		rts 

		lda #<$05e7			; $33c5 -> $05c5
		sta $30
		lda #>$05e7
		sta $31
-		ldy #$00			; $33cd -> $05cd
		lda ($30),y
		jsr $ffd2	;$ffd2 - output character                 
		inc $30
		bne +
		inc $31
+		lda $31				; $33da -> $05da
		cmp #$06
		bne -
		lda $30
		cmp #$82
		bne -
		rts

; ----------------------------------------
; De-compressor (copied to $0801 when file is saved)
; ----------------------------------------
; Basic header @ $0801
.byte $10,$08,$c6,$07,$9e,$32,$30,$36,$36,$20,$0c,$14,$08,$21,$00,$00,$00 ; $33e7 -> $0801


		ldx #$80 		; @$0812  (#2066 ;) 
-		lda $0820,x
		sta $043c,x
		dex				; $3400
		bpl -
		jmp $043c

		sei					; $0820 at decompressor routine running at $043c
		lda #$34
		sta $01
		ldy #$00
		sty $ac
		sty $ad
-		lda $ac			; $3411
		bne +
		dec $ad
+		dec $ac			; $3417
		lda $ae
		bne +
		dec $af
+		dec $ae			; $341f
		lda ($ae),y
		sta ($ac),y
		lda $ae
		cmp #$9c
		bne -
		lda $af
		cmp #$08
		bne -

		lda #<$0dc7
		sta $ae
		lda #>$0dc7
		sta $af
b3439	lda ($ac),y		; $3439
		cmp #$00
		bne +
		jsr $04aa
		lda ($ac),y
		tax 
		lda #$00
-		sta ($ae),y
		jsr $04b1
		dex 
		bne -
		beq b3467
+		cmp #$00		; $3451
		bne +
		jsr $04aa
		lda ($ac),y
		tax 
		jsr $04aa
		lda ($ac),y
		bne -
+		sta ($ae),y		; $3462
		jsr $04b1
b3467	jsr $04aa		; $3467
		bne b3439
		lda #$37
		sta $01
		cli 
		jmp Main	;$2801 decompression completed, thus let's launch the noter again ;)

		inc $ac
		bne +
		inc $ad
+		rts 

		inc $ae
		bne +
		inc $af
+		rts 

; this is most probably the table for converting to PETSCII (when saving filename)
.byte $10,$00,$30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$41,$42,$43,$44
.byte $45,$46,$14,$0d,$47,$48,$49,$4a,$4b,$4c,$4d,$4e,$4f,$50,$51,$52
.byte $53,$54,$55,$56,$57,$58,$59,$5a,$20,$21,$23,$24,$25,$26,$27,$28
.byte $29,$2b,$2d,$3b,$3a,$2e,$2c,$2f,$3f,$3e,$3c,$5c,$5b,$5d,$00,$10
.byte $11,$12,$13,$14,$15,$16,$17,$18,$19,$00,$01,$02,$03,$04,$05,$3c
.byte $20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20
	.byte $00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00

; not sure if this is called... :)
	lda #$ca			; $3560 -> $0760
	sta $0326
	lda #$f1
	sta $0327
	jmp $0340

.byte $00,$00,$20
		
.text "lethargy noter v1.0b by visage/lethargy!lethargy rules! vote for us in all mags!"
.text "save filename :"
.text "                          "

